// This file is derived form gmshEdge.cxx form gmsh source code and it
// is distributed on the same license as gmsh.

#include <sstream>
#include <assert.h>
#include "myEdge.h"

myEdge::myEdge(GModel *m, int tag, GVertex *v1, GVertex *v2)
  : GEdge(m, tag, v1, v2) {
}

Range<double> myEdge::parBounds(int i) const { 
 return Range<double>(0.0, 1.0);
}

GPoint myEdge::point(double par) const {
  assert( par >=0.0);
  assert( par <=1.0);
  double x = (1-par) * v0->x() + par * v1->x(); 
  double y = (1-par) * v0->y() + par * v1->y(); 
  double z = (1-par) * v0->z() + par * v1->z(); 
  return GPoint(x,y,z, this, par);
}

SVector3 myEdge::firstDer(double par) const {
  double x = - v0->x() +  v1->x(); 
  double y = - v0->y() +  v1->y(); 
  double z = - v0->z() +  v1->z(); 
  return SVector3(x,y,z);
}

void myEdge::writeGEO(FILE *fp) {
  fprintf(fp, "Line(%d) = ", tag());
  fprintf(fp, "{%d,%d}\n", v0->tag(), v1->tag());
}
