// This file is derived form gmshFace.cxx form gmsh source code and it
// is distributed on the same license as gmsh.

#include <stdlib.h>
#include "myFace.h"

myFace::myFace(GModel *m, int tag, const std::list<GEdge*> &edges )
  : GFace(m, tag) {
  edgeLoops.push_back(GEdgeLoop(edges));
  typedef std::list<GEdge*>::const_iterator EIter;
  for (EIter ei=edges.begin(); ei != edges.end(); ++ei) {; 
    GEdge *e = *ei;
    l_edges.push_back(e);
    e->addFace(this);
    l_dirs.push_back(1);
  }

  computeMeanPlane();

}

Range<double> myFace::parBounds(int i) const
{ 
  return Range<double>(0, 1);
}

Pair<SVector3, SVector3> myFace::firstDer(const SPoint2 &param) const
{
  double x, y, z, VX[3], VY[3];
  getMeanPlaneData(VX, VY, x, y, z);
  return Pair<SVector3, SVector3>(SVector3(VX[0], VX[1], VX[2]),
                                  SVector3(VY[0], VY[1], VY[2]));
}

void myFace::secondDer(const SPoint2 &param, 
                         SVector3 *dudu, SVector3 *dvdv, SVector3 *dudv) const
{
    *dudu = SVector3(0., 0., 0.);
    *dvdv = SVector3(0., 0., 0.);
    *dudv = SVector3(0., 0., 0.);
}

GPoint myFace::point(double par1, double par2) const {
  double pp[2] = {par1, par2};
  double x, y, z, VX[3], VY[3];
  getMeanPlaneData(VX, VY, x, y, z);
  return GPoint(x + VX[0] * par1 + VY[0] * par2,
                  y + VX[1] * par1 + VY[1] * par2,
                  z + VX[2] * par1 + VY[2] * par2, this, pp);
}

GEntity::GeomType myFace::geomType() const {
  return Plane;
}
