/*
 * This file is a part of FEMDK, a Finite Element Method Development Kit.
 * FEMDK provides utilities of developing simulation codes base on various 
 * variants of finite element method.
 *
 * Project web page: https://femdk.l5.pk.edu.pl/femdk
 * 
 * Copyright (c) 2013 Cracow University of Technology 
 * 
 * This file is distributed on "Simplified BSD License" as approved by 
 * the OSI Board. For licensing terms see LICENSE.BSD file distributed 
 * with this file.
 */

/*
 * Author: Roman Putanowicz <R.Putanowicz AT L5.pk.edu.pl>
 * Created: Thu Sep 26 13:39:14 2013
 * Modified by:
 */

/** 
 *  FIXME : describe the purpose of this file here
 */
#include "squaremesh_config.h"
#include <stdlib.h> // for EXIT_SUCCESS
#include "squaremesh_options.h" 
#include <iostream>

#include "gmsh/Gmsh.h"
#include "gmsh/GModel.h"
#include "gmsh/GEdgeLoop.h"
#include "myVertex.h"
#include "myEdge.h"
#include "myFace.h"

#include "femdk/femdk_common.hxx"
#include "femdk/femdk_params.hxx"
class MyParams : public femdk::OutputParams,
                 public femdk::LoggerParams,
                 public femdk::MiscParams {
  public:
    MyParams() {}
    void setUp(void);
    void echo(std::ostream &out);
};

inline void MyParams::setUp(void) {
  femdk::OutputParams::setUp();
  femdk::LoggerParams::setUp();
  femdk::MiscParams::setUp();
}

void MyParams::echo(std::ostream &out) { 
  femdk::OutputParams::echo(out); 
  femdk::LoggerParams::echo(out);
  femdk::MiscParams::echo(out);
}

int main_body(int argc, char **argv, MyParams &options);

/** build geometric model for square domain */
bool build_square(GModel *m);

FEMDK_STANDARD_MAIN(MyParams, squaremesh, true);

/** The function where the main work is done after configuration
 * and command line parsing.
 */
int main_body(int argc, char **argv, MyParams &options) {
  MYLOGGER().info("Starting program squaremesh");
  int result = EXIT_SUCCESS;

  GmshInitialize(argc, argv);
  GModel *m = new GModel;

  if (true == build_square(m)) { /* build square geometry */
    m->mesh(2);
    m->writeGEO("squaremesh.geo");
    m->writeMSH("squaremesh.msh");
  }

  delete m;
  GmshFinalize();

  return result;
}

bool build_square(GModel *m) {

  double L=1.0;
  std::vector<GVertex*> vertices;
  
  for (int j=0; j<2; j++) {
    for (int i=0; i<2; i++) {
      int tag = 2*j+i;
      GVertex *v = new myVertex(m, tag, 0.1);
      assert(NULL != v);
      GPoint p(L*i, L*j);
      v->setPosition(p);
      std::cout << v->x() << " " << v->y() << "\n";
      m->add(v);
      vertices.push_back(v);
    }
  } 
  std::list<GEdge*> edges;
  int vi[] = {0,1,3,2,0};
  for (int i=0; i<4; i++) {
    GEdge *e = new myEdge(m, i, vertices[vi[i]], vertices[vi[i+1]]);
    m->add(e);
    edges.push_back(e);
  }
  
  myFace *f = new myFace(m, 0, edges);
  m->add(f);

  return true;
}
